@@ -11,9 +11,9 @@
  *	under the terms of the GNU General Public License as published by the
  *	Free Software Foundation, version 2.
  *
- * see Documentation/dvb/README.dvb-usb for more information
+ * see Documentation/media/dvb-drivers/dvb-usb.rst for more information
  */
-#include "dvb-usb-ids.h"
+#include <media/dvb-usb-ids.h>
 #include "dw2102.h"
 #include "si21xx.h"
 #include "stv0299.h"
@@ -61,9 +61,7 @@
 #define P1100_FIRMWARE  "/*(DEBLOBBED)*/"
 #define P7500_FIRMWARE  "/*(DEBLOBBED)*/"
 
-#define	err_str "did not find the firmware file. (%s) " \
-		"Please see linux/Documentation/dvb/ for more details " \
-		"on firmware-problems."
+#define	err_str "did not find the firmware file '%s'. /*(DEBLOBBED)*/"
 
 struct dw2102_state {
 	u8 initialized;
